package app.fedilab.fedilabtube.fragment;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.ArrayList;
import java.util.List;

import app.fedilab.fedilabtube.R;
import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.HttpsConnection;
import app.fedilab.fedilabtube.client.PeertubeAPI;
import app.fedilab.fedilabtube.client.entities.Account;
import app.fedilab.fedilabtube.client.entities.ChannelCreation;
import app.fedilab.fedilabtube.drawer.AccountsListAdapter;
import app.fedilab.fedilabtube.helper.Helper;
import app.fedilab.fedilabtube.viewmodel.AccountsVM;
import es.dmoral.toasty.Toasty;


public class DisplayAccountsFragment extends Fragment implements AccountsListAdapter.AllAccountsRemoved, AccountsListAdapter.EditAlertDialog {

    private boolean flag_loading;
    private Context context;
    private AccountsListAdapter accountsListAdapter;
    private String max_id;
    private List<Account> accounts;
    private RelativeLayout mainLoader, nextElementLoader, textviewNoAction;
    private boolean firstLoad;
    private SwipeRefreshLayout swipeRefreshLayout;
    private String name;
    private boolean swiped;
    private RecyclerView lv_accounts;
    private View rootView;
    private AccountsVM.accountFetch accountFetch;
    private FloatingActionButton action_button;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

        rootView = inflater.inflate(R.layout.fragment_recyclerview, container, false);

        context = getContext();
        Bundle bundle = this.getArguments();
        accounts = new ArrayList<>();
        if (bundle != null) {
            if (bundle.containsKey("accountFetch")) {
                accountFetch = (AccountsVM.accountFetch) bundle.getSerializable("accountFetch");
            }
            name = bundle.getString("name", null);
        }
        max_id = null;
        firstLoad = true;
        flag_loading = true;
        swiped = false;

        swipeRefreshLayout = rootView.findViewById(R.id.swipeContainer);


        if (getActivity() != null) {
            action_button = getActivity().findViewById(R.id.action_button);
            action_button.setOnClickListener(view -> manageAlert(null));
        }
        lv_accounts = rootView.findViewById(R.id.lv_elements);
        lv_accounts.addItemDecoration(new DividerItemDecoration(context, DividerItemDecoration.VERTICAL));
        mainLoader = rootView.findViewById(R.id.loader);
        nextElementLoader = rootView.findViewById(R.id.loading_next);
        textviewNoAction = rootView.findViewById(R.id.no_action);
        mainLoader.setVisibility(View.VISIBLE);
        nextElementLoader.setVisibility(View.GONE);
        accountsListAdapter = new AccountsListAdapter(accountFetch, this.accounts);
        accountsListAdapter.allAccountsRemoved = this;
        accountsListAdapter.editAlertDialog = this;
        lv_accounts.setAdapter(accountsListAdapter);
        TextView no_action_text = rootView.findViewById(R.id.no_action_text);
        if (accountFetch == AccountsVM.accountFetch.MUTED) {
            no_action_text.setText(context.getString(R.string.no_muted));
        }
        final LinearLayoutManager mLayoutManager;
        mLayoutManager = new LinearLayoutManager(context);
        lv_accounts.setLayoutManager(mLayoutManager);
        lv_accounts.addOnScrollListener(new RecyclerView.OnScrollListener() {
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                if (dy > 0) {
                    int visibleItemCount = mLayoutManager.getChildCount();
                    int totalItemCount = mLayoutManager.getItemCount();
                    int firstVisibleItem = mLayoutManager.findFirstVisibleItemPosition();
                    if (firstVisibleItem + visibleItemCount == totalItemCount) {
                        if (!flag_loading) {
                            flag_loading = true;
                            AccountsVM viewModel = new ViewModelProvider(DisplayAccountsFragment.this).get(AccountsVM.class);
                            if (accountFetch == null) {
                                viewModel.getAccounts(null, name, AccountsVM.accountFetch.SINGLE_ACCOUNT).observe(DisplayAccountsFragment.this.requireActivity(), apiResponse -> manageViewAccounts(apiResponse));
                            } else {
                                String param = accountFetch == AccountsVM.accountFetch.CHANNEL ? name : max_id;
                                viewModel.getAccounts(param, null, accountFetch).observe(DisplayAccountsFragment.this.requireActivity(), apiResponse -> manageViewAccounts(apiResponse));
                            }
                            nextElementLoader.setVisibility(View.VISIBLE);
                        }
                    } else {
                        nextElementLoader.setVisibility(View.GONE);
                    }
                }
            }
        });
        swipeRefreshLayout.setOnRefreshListener(this::pullToRefresh);
        AccountsVM viewModel = new ViewModelProvider(this).get(AccountsVM.class);
        if (accountFetch == null) {
            viewModel.getAccounts(null, name, AccountsVM.accountFetch.SINGLE_ACCOUNT).observe(DisplayAccountsFragment.this.requireActivity(), this::manageViewAccounts);
        } else {
            String param = accountFetch == AccountsVM.accountFetch.CHANNEL ? name : null;
            viewModel.getAccounts(param, null, accountFetch).observe(DisplayAccountsFragment.this.requireActivity(), this::manageViewAccounts);
        }
        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();
        if (accountFetch == AccountsVM.accountFetch.CHANNEL) {
            action_button.setVisibility(View.VISIBLE);
        } else {
            action_button.setVisibility(View.GONE);
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        rootView = null;
    }

    @Override
    public void onCreate(Bundle saveInstance) {
        super.onCreate(saveInstance);
    }


    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        this.context = context;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

    public void scrollToTop() {
        if (lv_accounts != null)
            lv_accounts.setAdapter(accountsListAdapter);
    }

    private void manageViewAccounts(APIResponse apiResponse) {
        mainLoader.setVisibility(View.GONE);
        nextElementLoader.setVisibility(View.GONE);
        if (apiResponse.getError() != null) {
            Toasty.error(context, apiResponse.getError().getError(), Toast.LENGTH_LONG).show();
            swipeRefreshLayout.setRefreshing(false);
            swiped = false;
            flag_loading = false;
            return;
        }
        flag_loading = (apiResponse.getMax_id() == null);
        List<Account> accounts = apiResponse.getAccounts();
        if (!swiped && firstLoad && (accounts == null || accounts.size() == 0))
            textviewNoAction.setVisibility(View.VISIBLE);
        else
            textviewNoAction.setVisibility(View.GONE);

        max_id = apiResponse.getMax_id();
        if (swiped) {
            accountsListAdapter = new AccountsListAdapter(accountFetch, this.accounts);
            lv_accounts.setAdapter(accountsListAdapter);
            swiped = false;
        }
        if (accounts != null && accounts.size() > 0) {
            int currentPosition = this.accounts.size();
            this.accounts.addAll(accounts);
            accountsListAdapter.notifyItemRangeChanged(currentPosition, accounts.size());
        }
        swipeRefreshLayout.setRefreshing(false);
        firstLoad = false;
    }

    public void pullToRefresh() {
        max_id = null;
        accounts = new ArrayList<>();
        firstLoad = true;
        flag_loading = true;
        swiped = true;
        swipeRefreshLayout.setRefreshing(true);
        AccountsVM viewModel = new ViewModelProvider(this).get(AccountsVM.class);
        if (accountFetch == null) {
            viewModel.getAccounts(null, name, AccountsVM.accountFetch.SINGLE_ACCOUNT).observe(DisplayAccountsFragment.this.requireActivity(), this::manageViewAccounts);
        } else {
            String param = accountFetch == AccountsVM.accountFetch.CHANNEL ? name : null;
            viewModel.getAccounts(param, null, accountFetch).observe(DisplayAccountsFragment.this.requireActivity(), this::manageViewAccounts);
        }
    }

    @Override
    public void onAllAccountsRemoved() {
        textviewNoAction.setVisibility(View.VISIBLE);
    }


    public void manageAlert(ChannelCreation oldChannelValues) {

        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(context);
        LayoutInflater inflater1 = ((Activity) context).getLayoutInflater();
        View dialogView = inflater1.inflate(R.layout.add_channel, new LinearLayout(context), false);
        dialogBuilder.setView(dialogView);
        EditText display_name = dialogView.findViewById(R.id.display_name);
        EditText name = dialogView.findViewById(R.id.name);
        EditText description = dialogView.findViewById(R.id.description);
        if (oldChannelValues != null) {
            display_name.setText(oldChannelValues.getDisplayName());
            name.setText(oldChannelValues.getName());
            description.setText(oldChannelValues.getDescription());
            name.setEnabled(false);
        }
        dialogBuilder.setPositiveButton(R.string.validate, null);
        dialogBuilder.setNegativeButton(R.string.cancel, (dialog, id) -> dialog.dismiss());

        AlertDialog alertDialog = dialogBuilder.create();

        alertDialog.setOnShowListener(dialogInterface -> {

            Button button = alertDialog.getButton(AlertDialog.BUTTON_POSITIVE);
            button.setOnClickListener(view -> {
                if (display_name.getText() != null && display_name.getText().toString().trim().length() > 0 && name.getText() != null && name.getText().toString().trim().length() > 0) {

                    ChannelCreation channelCreation = new ChannelCreation();
                    channelCreation.setDisplayName(display_name.getText().toString().trim());
                    channelCreation.setName(name.getText().toString().trim());
                    if (description.getText() != null && description.getText().toString().trim().length() > 0) {
                        channelCreation.setDescription(description.getText().toString().trim());
                    }
                    new Thread(() -> {
                        try {
                            if (oldChannelValues == null) {
                                new PeertubeAPI(context).createChannel(channelCreation);
                            } else {
                                new PeertubeAPI(context).updateChannel(channelCreation.getName() + "@" + Helper.getLiveInstance(context), channelCreation);
                            }
                            Handler mainHandler = new Handler(Looper.getMainLooper());
                            Runnable myRunnable = () -> {

                                if (getActivity() == null)
                                    return;
                                if (oldChannelValues == null) {
                                    Account channel = new Account();
                                    channel.setAcct(channelCreation.getName() + "@" + Helper.getLiveInstance(context));
                                    channel.setUsername(channelCreation.getName());
                                    channel.setDisplay_name(channelCreation.getDisplayName());
                                    channel.setNote(channelCreation.getDescription());
                                    accounts.add(0, channel);
                                    accountsListAdapter.notifyItemInserted(0);
                                } else {
                                    int position = 0;
                                    for (Account account : accounts) {
                                        if (account.getId().compareTo(oldChannelValues.getId()) == 0) {
                                            account.setNote(channelCreation.getDescription());
                                            account.setDisplay_name(channelCreation.getDisplayName());
                                            break;
                                        }
                                        position++;
                                    }
                                    accountsListAdapter.notifyItemChanged(position);
                                }
                                action_button.setEnabled(true);
                            };
                            mainHandler.post(myRunnable);
                        } catch (HttpsConnection.HttpsConnectionException e) {

                            e.printStackTrace();
                            Handler mainHandler = new Handler(Looper.getMainLooper());
                            Runnable myRunnable = () -> {
                                action_button.setEnabled(true);
                                if (e.getMessage() != null) {
                                    Toasty.error(context, e.getMessage(), Toast.LENGTH_LONG).show();
                                } else {
                                    Toasty.error(context, context.getString(R.string.toast_error), Toast.LENGTH_LONG).show();
                                }
                            };
                            mainHandler.post(myRunnable);
                        }
                    }).start();
                    alertDialog.dismiss();
                    action_button.setEnabled(false);
                } else {
                    Toasty.error(context, context.getString(R.string.error_display_name_channel), Toast.LENGTH_LONG).show();
                }
            });
        });
        if (oldChannelValues == null) {
            alertDialog.setTitle(getString(R.string.action_channel_create));
        } else {
            alertDialog.setTitle(getString(R.string.action_channel_edit));
        }
        alertDialog.setOnDismissListener(dialogInterface -> {
            //Hide keyboard
            InputMethodManager imm = (InputMethodManager) context.getSystemService(Context.INPUT_METHOD_SERVICE);
            assert imm != null;
            imm.hideSoftInputFromWindow(display_name.getWindowToken(), 0);
        });
        if (alertDialog.getWindow() != null)
            alertDialog.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);
        alertDialog.show();
    }

    @Override
    public void show(Account account) {
        ChannelCreation oldChannelValues = new ChannelCreation();
        oldChannelValues.setName(account.getUsername());
        oldChannelValues.setDescription(account.getNote());
        oldChannelValues.setDisplayName(account.getDisplay_name());
        oldChannelValues.setId(account.getId());
        manageAlert(oldChannelValues);
    }
}
