package app.fedilab.fedilabtube.viewmodel;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.PeertubeAPI;


public class AccountsVM extends AndroidViewModel {
    private MutableLiveData<APIResponse> apiResponseMutableLiveData;

    public AccountsVM(@NonNull Application application) {
        super(application);
    }

    public LiveData<APIResponse> getAccounts(String max_id, String name, accountFetch type) {
        apiResponseMutableLiveData = new MutableLiveData<>();
        loadAccounts(max_id, name, type);
        return apiResponseMutableLiveData;
    }

    private void loadAccounts(String max_id, String name, accountFetch type) {
        Context _mContext = getApplication().getApplicationContext();
        new Thread(() -> {
            try {
                PeertubeAPI peertubeAPI = new PeertubeAPI(_mContext);
                APIResponse apiResponse = null;
                if (type == accountFetch.SUBSCRIPTION) {
                    apiResponse = peertubeAPI.getSubscriptionUsers(max_id);
                } else if (type == accountFetch.MUTED) {
                    apiResponse = peertubeAPI.getMuted(max_id);
                } else if (type == accountFetch.CHANNEL) {
                    apiResponse = peertubeAPI.getPeertubeChannel(max_id);
                } else if (type == accountFetch.SINGLE_ACCOUNT) {
                    apiResponse = peertubeAPI.getPeertubeChannel(name);
                } else if (type == accountFetch.SINGLE_CHANNEL) {
                    apiResponse = peertubeAPI.getPeertubeChannelInfo(name);
                }
                Handler mainHandler = new Handler(Looper.getMainLooper());
                APIResponse finalApiResponse = apiResponse;
                Runnable myRunnable = () -> apiResponseMutableLiveData.setValue(finalApiResponse);
                mainHandler.post(myRunnable);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }).start();
    }

    public enum accountFetch {
        SUBSCRIPTION,
        CHANNEL,
        MUTED,
        SINGLE_ACCOUNT,
        SINGLE_CHANNEL,
        OWN_ACCOUNTS
    }
}
