package app.fedilab.fedilabtube.viewmodel;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.sqlite.SQLiteDatabase;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import java.util.List;

import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.PeertubeAPI;
import app.fedilab.fedilabtube.client.entities.Peertube;
import app.fedilab.fedilabtube.helper.Helper;
import app.fedilab.fedilabtube.sqlite.PeertubeFavoritesDAO;
import app.fedilab.fedilabtube.sqlite.Sqlite;


public class FeedsVM extends AndroidViewModel {
    private MutableLiveData<APIResponse> apiResponseMutableLiveData;

    public FeedsVM(@NonNull Application application) {
        super(application);
    }

    public LiveData<APIResponse> getVideos(Type action, String max_id, String target, String forAccount) {
        apiResponseMutableLiveData = new MutableLiveData<>();
        loadVideos(action, max_id, target, forAccount);
        return apiResponseMutableLiveData;
    }


    public LiveData<APIResponse> getVideo(String instanceName, String videoId) {
        apiResponseMutableLiveData = new MutableLiveData<>();
        getSingle(instanceName, videoId);
        return apiResponseMutableLiveData;
    }

    public LiveData<APIResponse> updateVideo(Peertube peertube) {
        apiResponseMutableLiveData = new MutableLiveData<>();
        update(peertube);
        return apiResponseMutableLiveData;
    }

    private void update(Peertube peertube) {
        Context _mContext = getApplication().getApplicationContext();
        new Thread(() -> {
            try {
                PeertubeAPI peertubeAPI = new PeertubeAPI(_mContext);
                APIResponse apiResponse = peertubeAPI.updateVideo(peertube);
                if (apiResponse != null && apiResponse.getPeertubes() != null && apiResponse.getPeertubes().size() > 0)
                    apiResponse.getPeertubes().get(0).setUpdate(true);
                Handler mainHandler = new Handler(Looper.getMainLooper());
                Runnable myRunnable = () -> apiResponseMutableLiveData.postValue(apiResponse);
                mainHandler.post(myRunnable);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }).start();
    }


    private void getSingle(String instanceName, String videoId) {
        Context _mContext = getApplication().getApplicationContext();
        new Thread(() -> {
            try {
                PeertubeAPI peertubeAPI = new PeertubeAPI(_mContext);
                SharedPreferences sharedpreferences = _mContext.getSharedPreferences(Helper.APP_PREFS, Context.MODE_PRIVATE);
                String token = sharedpreferences.getString(Helper.PREF_KEY_OAUTH_TOKEN, null);
                APIResponse apiResponse = peertubeAPI.getSinglePeertube(instanceName, videoId, token);
                if (apiResponse.getPeertubes() != null && apiResponse.getPeertubes().size() > 0 && apiResponse.getPeertubes().get(0) != null) {
                    String rate = new PeertubeAPI(_mContext).getRating(videoId);
                    if (rate != null)
                        apiResponse.getPeertubes().get(0).setMyRating(rate);
                }
                Handler mainHandler = new Handler(Looper.getMainLooper());
                Runnable myRunnable = () -> apiResponseMutableLiveData.setValue(apiResponse);
                mainHandler.post(myRunnable);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }).start();
    }

    private void loadVideos(Type action, String max_id, String target, String forAccount) {
        Context _mContext = getApplication().getApplicationContext();
        new Thread(() -> {
            try {
                PeertubeAPI peertubeAPI = new PeertubeAPI(_mContext);
                if (action == null)
                    return;
                APIResponse apiResponse = null;
                switch (action) {
                    case USER:
                        apiResponse = peertubeAPI.getVideos(target, max_id);
                        break;
                    case MYVIDEOS:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getMyVideos(max_id);
                        break;
                    case PEERTUBE_HISTORY:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getMyHistory(max_id);
                        break;
                    case CHANNEL:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getVideosChannel(target, max_id);
                        break;
                    case CACHE_BOOKMARKS_PEERTUBE:
                        apiResponse = new APIResponse();
                        SQLiteDatabase db = Sqlite.getInstance(_mContext, Sqlite.DB_NAME, null, Sqlite.DB_VERSION).open();
                        List<Peertube> peertubes = new PeertubeFavoritesDAO(_mContext, db).getAllPeertube();
                        apiResponse.setPeertubes(peertubes);
                        break;
                    case PSUBSCRIPTIONS:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        if (forAccount == null) {
                            apiResponse = peertubeAPI.getSubscriptionsTL(max_id);
                        } else {
                            apiResponse = peertubeAPI.getVideosChannel(forAccount, max_id);
                        }
                        break;
                    case POVERVIEW:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getOverviewTL(max_id);
                        break;
                    case PTRENDING:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getTrendingTL(max_id);
                        break;
                    case PRECENTLYADDED:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getRecentlyAddedTL(max_id);
                        break;
                    case PLOCAL:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getLocalTL(max_id);
                        break;
                    case PPUBLIC:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getPublicTL(max_id);
                        break;
                    case PMOSTLIKED:
                        peertubeAPI = new PeertubeAPI(_mContext);
                        apiResponse = peertubeAPI.getLikedTL(max_id);
                        break;
                }
                Handler mainHandler = new Handler(Looper.getMainLooper());
                APIResponse finalApiResponse = apiResponse;
                Runnable myRunnable = () -> apiResponseMutableLiveData.setValue(finalApiResponse);
                mainHandler.post(myRunnable);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }).start();
    }

    public enum Type {

        USER,
        PPUBLIC,
        PSUBSCRIPTIONS,
        POVERVIEW,
        PTRENDING,
        PRECENTLYADDED,
        PMOSTLIKED,
        PLOCAL,
        CHANNEL,
        MYVIDEOS,
        PEERTUBE_HISTORY,
        CACHE_BOOKMARKS_PEERTUBE,
    }
}
