package app.fedilab.fedilabtube.viewmodel;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.sqlite.SQLiteDatabase;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import java.util.ArrayList;

import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.PeertubeAPI;
import app.fedilab.fedilabtube.client.entities.Account;
import app.fedilab.fedilabtube.client.entities.Playlist;
import app.fedilab.fedilabtube.helper.Helper;
import app.fedilab.fedilabtube.sqlite.AccountDAO;
import app.fedilab.fedilabtube.sqlite.Sqlite;


public class PlaylistsVM extends AndroidViewModel {
    private MutableLiveData<APIResponse> apiResponseMutableLiveData;

    public PlaylistsVM(@NonNull Application application) {
        super(application);
    }

    public LiveData<APIResponse> manage(action apiAction, Playlist playlist, String videoId, String max_id) {
        apiResponseMutableLiveData = new MutableLiveData<>();
        managePlaylists(apiAction, playlist, videoId, max_id);
        return apiResponseMutableLiveData;
    }

    private void managePlaylists(action apiAction, Playlist playlist, String videoId, String max_id) {
        Context _mContext = getApplication().getApplicationContext();
        new Thread(() -> {
            try {
                SharedPreferences sharedpreferences = _mContext.getSharedPreferences(Helper.APP_PREFS, Context.MODE_PRIVATE);
                String userId = sharedpreferences.getString(Helper.PREF_KEY_ID, null);
                String instance = Helper.getLiveInstance(_mContext);
                SQLiteDatabase db = Sqlite.getInstance(_mContext.getApplicationContext(), Sqlite.DB_NAME, null, Sqlite.DB_VERSION).open();
                Account account = new AccountDAO(_mContext, db).getUniqAccount(userId, instance);
                if (account == null) {
                    account = new AccountDAO(_mContext, db).getUniqAccount(userId, Helper.getPeertubeUrl(instance));
                }
                int statusCode = -1;
                APIResponse apiResponse = null;
                if (account == null) {
                    statusCode = 403;
                    apiResponse = new APIResponse();
                    apiResponse.setPlaylists(new ArrayList<>());
                } else if (apiAction == action.GET_PLAYLIST) {
                    apiResponse = new PeertubeAPI(_mContext).getPlayists(account.getUsername());
                } else if (apiAction == action.GET_LIST_VIDEOS) {
                    apiResponse = new PeertubeAPI(_mContext).getPlaylistVideos(playlist.getId(), max_id, null);
                } else if (apiAction == action.DELETE_PLAYLIST) {
                    statusCode = new PeertubeAPI(_mContext).deletePlaylist(playlist.getId());
                } else if (apiAction == action.ADD_VIDEOS) {
                    statusCode = new PeertubeAPI(_mContext).addVideoPlaylist(playlist.getId(), videoId);
                } else if (apiAction == action.DELETE_VIDEOS) {
                    statusCode = new PeertubeAPI(_mContext).deleteVideoPlaylist(playlist.getId(), videoId);
                } else if (apiAction == action.GET_PLAYLIST_FOR_VIDEO) {
                    apiResponse = new PeertubeAPI(_mContext).getPlaylistForVideo(videoId);
                }
                Handler mainHandler = new Handler(Looper.getMainLooper());
                if (apiResponse != null) {
                    apiResponse.setStatusCode(statusCode);
                }
                APIResponse finalApiResponse = apiResponse;
                Runnable myRunnable = () -> apiResponseMutableLiveData.setValue(finalApiResponse);
                mainHandler.post(myRunnable);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }).start();
    }

    public enum action {
        GET_PLAYLIST,
        GET_LIST_VIDEOS,
        CREATE_PLAYLIST,
        DELETE_PLAYLIST,
        ADD_VIDEOS,
        DELETE_VIDEOS,
        GET_PLAYLIST_FOR_VIDEO,
    }
}
