package app.fedilab.fedilabtube.viewmodel;
/* Copyright 2020 Thomas Schneider
 *
 * This file is a part of TubeLab
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * TubeLab is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with TubeLab; if not,
 * see <http://www.gnu.org/licenses>. */

import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.lifecycle.AndroidViewModel;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import app.fedilab.fedilabtube.client.APIResponse;
import app.fedilab.fedilabtube.client.PeertubeAPI;


public class PostActionsVM extends AndroidViewModel {
    private MutableLiveData<APIResponse> apiResponseMutableLiveData;

    public PostActionsVM(@NonNull Application application) {
        super(application);
    }

    public LiveData<APIResponse> post(PeertubeAPI.StatusAction apiAction, String targetedId, String comment, String targetedComment) {
        apiResponseMutableLiveData = new MutableLiveData<>();
        makeAction(apiAction, targetedId, comment, targetedComment);
        return apiResponseMutableLiveData;
    }

    private void makeAction(PeertubeAPI.StatusAction apiAction, String targetedId, String comment, String targetedComment) {
        Context _mContext = getApplication().getApplicationContext();
        new Thread(() -> {
            try {
                PeertubeAPI peertubeAPI = new PeertubeAPI(_mContext);
                int statusCode = -1;
                if (apiAction == PeertubeAPI.StatusAction.FOLLOW || apiAction == PeertubeAPI.StatusAction.UNFOLLOW
                        || apiAction == PeertubeAPI.StatusAction.MUTE || apiAction == PeertubeAPI.StatusAction.UNMUTE) {
                    statusCode = peertubeAPI.postAction(apiAction, targetedId);
                } else if (apiAction == PeertubeAPI.StatusAction.RATEVIDEO)
                    statusCode = peertubeAPI.postRating(targetedId, comment);
                else if (apiAction == PeertubeAPI.StatusAction.PEERTUBECOMMENT)
                    statusCode = peertubeAPI.postComment(targetedId, comment);
                else if (apiAction == PeertubeAPI.StatusAction.PEERTUBEREPLY)
                    statusCode = peertubeAPI.postReply(targetedId, comment, targetedComment);
                else if (apiAction == PeertubeAPI.StatusAction.PEERTUBEDELETECOMMENT) {
                    statusCode = peertubeAPI.deleteComment(targetedId, comment);
                } else if (apiAction == PeertubeAPI.StatusAction.PEERTUBEDELETEVIDEO) {
                    statusCode = peertubeAPI.deleteVideo(targetedId);
                } else if (apiAction == PeertubeAPI.StatusAction.REPORT_ACCOUNT) {
                    statusCode = peertubeAPI.report(PeertubeAPI.reportType.ACCOUNT, targetedId, comment);
                } else if (apiAction == PeertubeAPI.StatusAction.REPORT_VIDEO) {
                    statusCode = peertubeAPI.report(PeertubeAPI.reportType.VIDEO, targetedId, comment);
                }
                APIResponse apiResponse = new APIResponse();
                apiResponse.setError(peertubeAPI.getError());
                apiResponse.setStatusCode(statusCode);
                apiResponse.setTargetedId(targetedId);
                Handler mainHandler = new Handler(Looper.getMainLooper());
                Runnable myRunnable = () -> apiResponseMutableLiveData.setValue(apiResponse);
                mainHandler.post(myRunnable);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }).start();
    }

}
